@@ -309,7 +309,8 @@ int ftp_open_url(url_t *urlp, bool reset_vars)
     if (!urlp->protocol || strcmp(urlp->protocol, "ssh") != 0) {
       // We only need to do a host lookup if we're not using sftp.
       if(!host_lookup(ftp->host)) {
-        herror(host_getname(ftp->host));
+        fprintf(stderr, "Host look up for '%s' failed: %s\n",
+            host_getname(ftp->host), host_geterror(ftp->host));
         alarm(0);
         ftp_set_signal(SIGALRM, SIG_IGN);
         return -1;
@@ -354,7 +355,7 @@ int ftp_open_url(url_t *urlp, bool reset_vars)
     }
 
     ftp->ctrl = sock_create();
-    if (ftp->ctrl == 0) {
+    if (!ftp->ctrl) {
         ftp_err(_("Unable to create socket.\n"));
         alarm(0);
         ftp_set_signal(SIGALRM, SIG_IGN);
@@ -434,8 +435,6 @@ int ftp_reopen(void)
  */
 static int ftp_gets(void)
 {
-    int c, i=0;
-
     ftp->reply[0] = 0;
 
     if(!sock_connected(ftp->ctrl)) {
@@ -443,14 +442,29 @@ static int ftp_gets(void)
         return -1;
     }
 
-    while(true) {
-        c = sock_get(ftp->ctrl);
-        if(c == EOF) {
+    size_t i = 0;
+    bool next = true;
+    int nc = 0;
+    while(true)
+    {
+        int c = 0;
+        if (next)
+          c = sock_get(ftp->ctrl);
+        else
+        {
+          c = nc;
+          next = true;
+        }
+
+        if (c == EOF)
+        {
             ftp_err(_("Server has closed control connection\n"));
             ftp_close();
             return -1;
         }
-        else if(c == 255/*IAC*/) {   /* handle telnet commands */
+        else if(c == 255/*IAC*/)
+        {
+            /* handle telnet commands */
             switch(c = sock_get(ftp->ctrl)) {
               case 251/*WILL*/:
               case 252/*WONT*/:
@@ -469,21 +483,21 @@ static int ftp_gets(void)
             }
             continue;
         }
-        else if(c == '\r') {
-            c = sock_get(ftp->ctrl);
-            if(c == 0)
-                c = '\r';
-            else if(c == '\n') {
+        else if (c == '\r')
+        {
+            nc = sock_get(ftp->ctrl);
+            if (nc == 0)
+              ; // do nothing
+            else if (nc == '\n') {
                 ftp->reply[i++] = (char)c;
                 break;
-            } else if(c == EOF)
-                /* do nothing */ ;
-            else { /* telnet protocol violation, hmpf... */
-                sock_unget(ftp->ctrl, c);
-                continue;
-            }
+            } else if(nc == EOF)
+                c = EOF ;
+            else
+                /* telnet protocol violation, hmpf... */
+                next = false;
         }
-        else if(c == '\n')
+        else if (c == '\n')
             break;
         if(i < MAXREPLY)
             ftp->reply[i++] = (char)c;
@@ -584,11 +598,12 @@ int ftp_read_reply(void)
     if(ftp->reply_timeout)
         alarm(ftp->reply_timeout);
 
-    clearerr(sock_sin(ftp->ctrl));
+    sock_clearerr_in(ftp->ctrl);
     r = ftp_gets();
     if(!sock_connected(ftp->ctrl)) {
         alarm(0);
         ftp_set_signal(SIGALRM, SIG_DFL);
+        ftp_trace("sock is not connected\n");
         return -1;
     }
 
@@ -623,7 +638,10 @@ static void ftp_print_cmd(const char *cmd, va_list ap)
             ftp_err("PASS ********");
         else {
             vfprintf(stderr, cmd, ap);
-            ftp_vtrace(cmd, ap);
+            va_list aq;
+            va_copy(aq, ap);
+            ftp_vtrace(cmd, aq);
+            va_end(aq);
         }
         ftp_err("\n");
     } else {
@@ -662,7 +680,7 @@ int ftp_cmd(const char *cmd, ...)
     sock_flush(ftp->ctrl);
     va_end(ap);
 
-    if(ferror(sock_sout(ftp->ctrl))) {
+    if (sock_error_out(ftp->ctrl)) {
         ftp_err(_("error writing command"));
         ftp_err(" (");
         va_start(ap, cmd);
@@ -854,10 +872,8 @@ static const char *secext_name(const char *mech)
 
 static bool mech_unsupported(const char *mech)
 {
-#ifndef HAVE_KRB4
     if(strcasecmp(mech, "KERBEROS_V4") == 0)
         return true;
-#endif
 #ifndef HAVE_KRB5
     if(strcasecmp(mech, "GSSAPI") == 0)
         return true;
@@ -1072,6 +1088,7 @@ int ftp_login(const char *guessed_username, const char *anonpass)
                     level_to_name(ftp->data_prot));
         }
 #endif
+
         ftp->homedir = ftp_getcurdir();
         ftp->curdir = xstrdup(ftp->homedir);
         ftp->prevdir = xstrdup(ftp->homedir);
@@ -1537,7 +1554,7 @@ static bool ftp_path_part_of(const char *a, const char *b)
 int ftp_mkpath(const char *path)
 {
     bool one_created = false;
-    char *p, *orgp, *e = 0;
+    char *p, *orgp, *e = NULL;
 
     if(!path)
         return 0;
@@ -1560,11 +1577,11 @@ int ftp_mkpath(const char *path)
     }
 
     while(true) {
-        char *tmp, *foo;
-        tmp = strqsep(&p, '/');
+        char* tmp = strqsep(&p, '/');
         if(!tmp)
             break;
 
+        char* foo = NULL;
         if (e)
         {
             if (asprintf(&foo, "%s/%s", e, tmp) == -1)
@@ -1581,7 +1598,6 @@ int ftp_mkpath(const char *path)
         free(e);
         e = foo;
 
-
         /* check if we already has created this path */
         if(ftp_path_part_of(e, ftp->last_mkpath))
             continue;
@@ -1590,7 +1606,7 @@ int ftp_mkpath(const char *path)
         if(ftp_path_part_of(e, ftp->curdir))
             continue;
 
-        if(strcmp(e, ".") != 0) {
+        if(strncmp(e, ".", 2) != 0) {
             ftp_mkdir_verb(e, vbNone);
             one_created = (ftp->code == ctComplete);
         }
@@ -1611,9 +1627,6 @@ char *ftp_path_absolute(const char *path)
 
 void ftp_flush_reply(void)
 {
-    fd_set ready;
-    struct timeval poll;
-
     if(!ftp_connected())
         return;
 
@@ -1628,12 +1641,7 @@ void ftp_flush_reply(void)
 /*  ftp_reply_timeout(10);*/
 
     while(ftp_connected()) {
-        poll.tv_sec = 1;
-        poll.tv_usec = 0;
-        FD_ZERO(&ready);
-        int handle = sock_handle(ftp->ctrl);
-        FD_SET(handle, &ready);
-        if(select(handle+1, &ready, 0, 0, &poll) == 1)
+        if (sock_check_pending(ftp->ctrl, false) == 1)
             ftp_read_reply();
         else
             break;
@@ -1843,9 +1851,8 @@ void ftp_pwd(void)
 
 char *perm2string(int perm)
 {
-    char *attr = (char *)xmalloc(11);
-
-    strcpy(attr, "----------");
+    char* attr = xmalloc(11);
+    strncpy(attr, "----------", 11);
 
     if(S_ISDIR(perm))
         attr[0] = 'd';
