@@ -72,15 +72,15 @@ static bool is_reserved(struct sockaddr* sa)
 
 static bool ftp_pasv(bool ipv6, unsigned char* result, unsigned short* ipv6_port)
 {
-	if(!ftp->has_pasv_command) {
-		ftp_err(_("Host doesn't support passive mode\n"));
-		return false;
-	}
-	ftp_set_tmp_verbosity(vbNone);
+  if (!ftp->has_pasv_command) {
+    ftp_err(_("Host doesn't support passive mode\n"));
+    return false;
+  }
+  ftp_set_tmp_verbosity(vbNone);
 
-	/* request passive mode */
+  /* request passive mode */
   if (!ipv6)
-  	ftp_cmd("PASV");
+    ftp_cmd("PASV");
 #ifdef HAVE_IPV6
   else if (ipv6)
     ftp_cmd("EPSV");
@@ -88,43 +88,47 @@ static bool ftp_pasv(bool ipv6, unsigned char* result, unsigned short* ipv6_port
   else
     return false;
 
-	if(!ftp_connected())
-		return false;
+  if (!ftp_connected())
+    return false;
 
-	if(ftp->code != ctComplete) {
-		ftp_err(_("Unable to enter passive mode\n"));
-		if(ftp->code == ctError) /* no use try it again */
-			ftp->has_pasv_command = false;
-		return false;
-	}
+  if (ftp->code != ctComplete) {
+    ftp_err(_("Unable to enter passive mode\n"));
+    if(ftp->code == ctError) /* no use try it again */
+      ftp->has_pasv_command = false;
+    return false;
+  }
 
-	char* e = ftp->reply + 4;
-	while(!isdigit((int)*e))
-		e++;
+  const char* e = ftp_getreply(false);
+  while (e && !isdigit((int)*e))
+    e++;
+
+  if (!e)
+  {
+    ftp_err(_("Error parsing EPSV/PASV reply: '%s'\n"), ftp_getreply(false));
+    return false;
+  }
 
   if (!ipv6)
   {
-    int pa[6];
-	  if (sscanf(e, "%d,%d,%d,%d,%d,%d", &pa[0], &pa[1], &pa[2], &pa[3], &pa[4], &pa[5]) != 6)
+    if (sscanf(e, "%hhu,%hhu,%hhu,%hhu,%hhu,%hhu", &result[0], &result[1],
+          &result[2], &result[3], &result[4], &result[5]) != 6)
     {
-  		ftp_err(_("Error parsing PASV reply: '%s'\n"), ftp_getreply(false));
-	  	return false;
+      ftp_err(_("Error parsing PASV reply: '%s'\n"), ftp_getreply(false));
+      return false;
     }
-    for (int i = 0; i < 6; ++i)
-		  result[i] = (unsigned char)(pa[i] & 0xFF);
-	}
+  }
 #ifdef HAVE_IPV6
   else
   {
-    if (sscanf(e, "%hd", ipv6_port) != 1)
+    if (sscanf(e, "%hu", ipv6_port) != 1)
     {
       ftp_err(_("Error parsing EPSV reply: '%s'\n"), ftp_getreply(false));
       return false;
     }
   }
 #endif
-	
-	return true;
+
+  return true;
 }
 
 static bool ftp_is_passive(void)
@@ -136,31 +140,34 @@ static bool ftp_is_passive(void)
 
 static int ftp_init_transfer(void)
 {
-	struct sockaddr_storage sa;
-	unsigned char *a, *p;
+  if (!ftp_connected())
+    return -1;
 
-	if(!ftp_connected())
-		return -1;
+  if (!sock_dup(ftp->ctrl, &ftp->data))
+    return -1;
 
-	if (!(ftp->data = sock_create())) {
-		return -1;
-	}
-	sock_copy(ftp->data, ftp->ctrl);
-
-	if (ftp_is_passive())
+  if (ftp_is_passive())
   {
+    ftp_trace("Initializing passive connection.\n");
+
+    struct sockaddr_storage sa;
     memcpy(&sa, sock_remote_addr(ftp->ctrl), sizeof(struct sockaddr_storage));
 
-    unsigned char pac[6];
-    unsigned short ipv6_port;
-		if (!ftp_pasv(sa.ss_family != AF_INET, pac, &ipv6_port))
-			goto err1;
+    unsigned char pac[6] = { 0 };
+    unsigned short ipv6_port = { 0 };
+    if (!ftp_pasv(sa.ss_family != AF_INET, pac, &ipv6_port))
+    {
+      ftp_trace("PASV/EPSV failed.\n");
+      sock_destroy(ftp->data);
+      ftp->data = NULL;
+      return -1;
+    }
 
     socklen_t len = sizeof(struct sockaddr_in);
     if (sa.ss_family == AF_INET)
     {
       memcpy(&((struct sockaddr_in*)&sa)->sin_addr, pac, (size_t)4);
-		  memcpy(&((struct sockaddr_in*)&sa)->sin_port, pac+4, (size_t)2);
+      memcpy(&((struct sockaddr_in*)&sa)->sin_port, pac+4, (size_t)2);
     }
 #ifdef HAVE_IPV6
     else if (sa.ss_family == AF_INET6)
@@ -170,39 +177,49 @@ static int ftp_init_transfer(void)
     }
 #endif
     else
+    {
+      ftp_trace("Do not know how to handle family %d.\n", sa.ss_family);
+      sock_destroy(ftp->data);
+      ftp->data = NULL;
       return -1;
+    }
 
     struct sockaddr_storage tmp;
     memcpy(&tmp, sock_remote_addr(ftp->ctrl), sizeof(struct sockaddr_storage));
-		if (is_reserved((struct sockaddr*) &sa) ||
-			   is_multicast((struct sockaddr*) &sa)  ||
-			   (is_private((struct sockaddr*) &sa) != is_private((struct sockaddr*) &tmp)) ||
-			   (is_loopback((struct sockaddr*) &sa) != is_loopback((struct sockaddr*) &tmp)))
-		{
-			// Invalid address returned by PASV. Replace with address from control
-			// socket.
-			ftp_err(_("Address returned by PASV seems to be incorrect.\n"));
-			((struct sockaddr_in*)&sa)->sin_addr = ((struct sockaddr_in*)&tmp)->sin_addr;
-		}
-
-		if (!sock_connect_addr(ftp->data, (struct sockaddr*) &sa, len))
+    if (is_reserved((struct sockaddr*) &sa) ||
+         is_multicast((struct sockaddr*) &sa)  ||
+         (is_private((struct sockaddr*) &sa) != is_private((struct sockaddr*) &tmp)) ||
+         (is_loopback((struct sockaddr*) &sa) != is_loopback((struct sockaddr*) &tmp)))
     {
-      perror("connect()");
-			goto err1;
+      // Invalid address returned by PASV. Replace with address from control
+      // socket.
+      ftp_err(_("Address returned by PASV seems to be incorrect.\n"));
+      ((struct sockaddr_in*)&sa)->sin_addr = ((struct sockaddr_in*)&tmp)->sin_addr;
     }
-	} else {
+
+    if (!sock_connect_addr(ftp->data, (struct sockaddr*) &sa, len))
+    {
+      ftp_trace("Could not connect to address from PASV/EPSV.\n");
+      perror("connect()");
+      sock_destroy(ftp->data);
+      ftp->data = NULL;
+      return -1;
+    }
+  } else {
+    ftp_trace("Initializing active connection.\n");
+
     const struct sockaddr* local = sock_local_addr(ftp->data);
-		sock_listen(ftp->data, local->sa_family);
+    sock_listen(ftp->data, local->sa_family);
 
     if (local->sa_family == AF_INET)
     {
       struct sockaddr_in* tmp = (struct sockaddr_in*)local;
-  		a = (unsigned char *)&tmp->sin_addr;
-	  	p = (unsigned char *)&tmp->sin_port;
+      unsigned char* a = (unsigned char *)&tmp->sin_addr;
+      unsigned char* p = (unsigned char *)&tmp->sin_port;
 
-		  ftp_set_tmp_verbosity(vbError);
-		  ftp_cmd("PORT %d,%d,%d,%d,%d,%d",
-				  a[0], a[1], a[2], a[3], p[0], p[1]);
+      ftp_set_tmp_verbosity(vbError);
+      ftp_cmd("PORT %d,%d,%d,%d,%d,%d",
+          a[0], a[1], a[2], a[3], p[0], p[1]);
     }
 #ifdef HAVE_IPV6
     else if (local->sa_family == AF_INET6)
@@ -210,25 +227,29 @@ static int ftp_init_transfer(void)
       char* addr = printable_address(local);
 
       ftp_set_tmp_verbosity(vbError);
-      ftp_cmd("EPRT |2|%s|%u|", addr, ntohs(((struct sockaddr_in6*)local)->sin6_port));
+      ftp_cmd("EPRT |2|%s|%u", addr, ntohs(((struct sockaddr_in6*)local)->sin6_port));
       free(addr);
     }
 #endif
     else
-      goto err1;
+    {
+      ftp_trace("Do not know how to handle family %d.\n", local->sa_family);
+      sock_destroy(ftp->data);
+      ftp->data = NULL;
+      return -1;
+    }
 
-		if(ftp->code != ctComplete)
-			goto err1;
-	}
+    if(ftp->code != ctComplete)
+    {
+      ftp_trace("PORT/EPRT not successful\n");
+      sock_destroy(ftp->data);
+      ftp->data = NULL;
+      return -1;
+    }
+  }
 
-	sock_throughput(ftp->data);
-
-	return 0;
-
- err1:
-	sock_destroy(ftp->data);
-	ftp->data = 0;
-	return -1;
+  sock_throughput(ftp->data);
+  return 0;
 }
 
 int ftp_type(transfer_mode_t type)
@@ -255,11 +276,9 @@ static ftp_transfer_func foo_hookf = 0;
 
 /* abort routine originally from Cftp by Dieter Baron
  */
-int ftp_abort(FILE *fp)
+int ftp_abort(Socket* fp)
 {
 	char buf[4096];
-	fd_set ready;
-	struct timeval poll;
 
 #ifdef HAVE_LIBSSH
 	if(ftp->session)
@@ -272,14 +291,10 @@ int ftp_abort(FILE *fp)
 
 	ftp_set_close_handler();
 
-	poll.tv_sec = poll.tv_usec = 0;
-	FD_ZERO(&ready);
-  int handle = sock_handle(ftp->ctrl);
-	FD_SET(handle, &ready);
-	if(select(handle+1, &ready, 0, 0, &poll) == 1) {
+	if (sock_check_pending(fp, false) == 1) {
 		ftp_trace("There is data on the control channel, won't send ABOR\n");
 		/* read remaining bytes from connection */
-		while(fp && fread(buf, 1, 4096, fp) > 0)
+		while(fp && sock_read(fp, buf, sizeof(buf)) > 0)
 			/* LOOP */ ;
 		return 0;
 	}
@@ -303,7 +318,7 @@ int ftp_abort(FILE *fp)
 		ftp_trace("--> [%s] ABOR\n", ftp->url->hostname);
 
     /* read remaining bytes from connection */
-	while(fp && fread(buf, 1, 4096, fp) > 0)
+	while(fp && sock_read(fp, buf, sizeof(buf)) > 0)
 		/* LOOP */ ;
 
 	/* we expect a 426 or 226 reply here... */
@@ -320,23 +335,10 @@ int ftp_abort(FILE *fp)
 	return -1;
 }
 
-static int wait_for_data(FILE *fd, bool wait_for_read)
+static int wait_for_data(Socket* fp, bool wait_for_read)
 {
-	fd_set fds;
-	struct timeval tv;
-	int r;
-
-	/* watch fd to see if it has input */
-	FD_ZERO(&fds);
-	FD_SET(fileno(fd), &fds);
-	/* wait max 1 second */
-	tv.tv_sec = 10;
-	tv.tv_usec = 0;
-
-	if(wait_for_read)
-		r = select(fileno(fd)+1, &fds, 0, 0, &tv);
-	else /* wait for write */
-		r = select(fileno(fd)+1, 0, &fds, 0, &tv);
+  errno = 0;
+	const int r = sock_check_pending(fp, !wait_for_read);
 #if 0
 	if(r < 0 && errno == EINTR && gvSigStopReceived && ftp_sigints() == 0) {
 		gvSigStopReceived = false;
@@ -369,7 +371,7 @@ static int wait_for_input(void)
 {
 	int r;
 	do {
-		r = wait_for_data(sock_sin(ftp->data), true);
+		r = wait_for_data(ftp->data, true);
 		if(r == -1) {
 			if(errno == EINTR)
 				ftp->ti.interrupted = true;
@@ -386,7 +388,7 @@ static int wait_for_output(void)
 {
 	int r;
 	do {
-		r = wait_for_data(sock_sout(ftp->data), false);
+		r = wait_for_data(ftp->data, false);
 		if(r == -1)
 			return -1;
 		if(r == 0 && foo_hookf)
@@ -396,7 +398,7 @@ static int wait_for_output(void)
 	return 0;
 }
 
-static int maybe_abort(FILE *in, FILE *out)
+static int maybe_abort_in(Socket* in, FILE *out)
 {
 	unsigned int i = ftp_sigints();
 	ftp_set_close_handler();
@@ -406,8 +408,8 @@ static int maybe_abort(FILE *in, FILE *out)
 	if(ftp->ti.interrupted)
 		i++;
 
-	if(i > 0 || ferror(in) || ferror(out)) {
-		if(ferror(in)) {
+	if(i > 0 || sock_error_in(in) || ferror(out)) {
+		if(sock_error_in(in)) {
 			ftp_err(_("read error: %s\n"), strerror(errno));
 			ftp->ti.ioerror = true;
 		}
@@ -415,15 +417,37 @@ static int maybe_abort(FILE *in, FILE *out)
 			ftp_err(_("write error: %s\n"), strerror(errno));
 			ftp->ti.ioerror = true;
 		}
-		return ftp_abort(ftp->ti.transfer_is_put ? out : in);
+		return ftp_abort(in);
 	}
 	return 0;
 }
 
-static int FILE_recv_binary(FILE *in, FILE *out)
+static int maybe_abort_out(FILE *in, Socket *out)
+{
+	unsigned int i = ftp_sigints();
+	ftp_set_close_handler();
+
+	ftp->ti.finished = true;
+
+	if(ftp->ti.interrupted)
+		i++;
+
+	if(i > 0 || ferror(in) || sock_error_out(out)) {
+		if(ferror(in)) {
+			ftp_err(_("read error: %s\n"), strerror(errno));
+			ftp->ti.ioerror = true;
+		}
+		else if (sock_error_out(out)) {
+			ftp_err(_("write error: %s\n"), strerror(errno));
+			ftp->ti.ioerror = true;
+		}
+		return ftp_abort(out);
+	}
+	return 0;
+}
+
+static int FILE_recv_binary(Socket* in, FILE *out)
 {
-	size_t n;
-	char *buf;
 	time_t then = time(0) - 1;
 	time_t now;
 
@@ -433,22 +457,18 @@ static int FILE_recv_binary(FILE *in, FILE *out)
 		foo_hookf(&ftp->ti);
 	ftp->ti.begin = false;
 
-	clearerr(in);
+	sock_clearerr_in(in);
 	clearerr(out);
 
-	buf = (char *)xmalloc(FTP_BUFSIZ);
-	while(!feof(in)) {
-
+	char* buf = xmalloc(FTP_BUFSIZ);
+	while (!sock_eof(in)) {
 		if(wait_for_input() != 0) {
 			ftp_trace("wait_for_input() returned non-zero\n");
 			break;
 		}
-#ifdef SECFTP
-		n = sec_read(fileno(in), buf, FTP_BUFSIZ);
-#else
-		n = fread(buf, sizeof(char), FTP_BUFSIZ, in);
-#endif
-		if(n <= 0)
+
+    const ssize_t n = sock_read(in, buf, FTP_BUFSIZ);
+		if (n <= 0)
 			break;
 
 		if(ftp_sigints() > 0) {
@@ -473,13 +493,11 @@ static int FILE_recv_binary(FILE *in, FILE *out)
 	free(buf);
 	ftp_set_close_handler();
 
-	return maybe_abort(in, out);
+	return maybe_abort_in(in, out);
 }
 
-static int FILE_send_binary(FILE *in, FILE *out)
+static int FILE_send_binary(FILE *in, Socket *out)
 {
-	size_t n;
-	char *buf;
 	time_t then = time(0) - 1;
 	time_t now;
 
@@ -490,11 +508,11 @@ static int FILE_send_binary(FILE *in, FILE *out)
 	ftp->ti.begin = false;
 
 	clearerr(in);
-	clearerr(out);
+	sock_clearerr_out(out);
 
-	buf = (char *)xmalloc(FTP_BUFSIZ);
+	char* buf = xmalloc(FTP_BUFSIZ);
 	while(!feof(in)) {
-		n = fread(buf, sizeof(char), FTP_BUFSIZ, in);
+		ssize_t n = fread(buf, sizeof(char), FTP_BUFSIZ, in);
 		if(n <= 0)
 			break;
 
@@ -503,13 +521,9 @@ static int FILE_send_binary(FILE *in, FILE *out)
 
 		if(wait_for_output() != 0)
 			break;
-#ifdef SECFTP
-		if(sec_write(fileno(out), buf, n) != n)
-			break;
-#else
-		if(fwrite(buf, sizeof(char), n, out) != n)
-			break;
-#endif
+
+    if (sock_write(out, buf, n) != n)
+      break;
 		ftp->ti.size += n;
 		if(foo_hookf) {
 			now = time(0);
@@ -519,28 +533,14 @@ static int FILE_send_binary(FILE *in, FILE *out)
 			}
 		}
 	}
-#ifdef SECFTP
-	sec_fflush(out);
-#endif
-
+	sock_flush(out);
 	free(buf);
 
-	return maybe_abort(in, out);
+	return maybe_abort_out(in, out);
 }
 
-static int krb_getc(FILE *fp)
+static int FILE_recv_ascii(Socket* in, FILE *out)
 {
-#ifdef SECFTP
-	return sec_getc(fp);
-#else
-	return fgetc(fp);
-#endif
-}
-
-static int FILE_recv_ascii(FILE *in, FILE *out)
-{
-	char *buf = (char *)xmalloc(FTP_BUFSIZ);
-	int c;
 	time_t then = time(0) - 1;
 	time_t now;
 
@@ -550,28 +550,42 @@ static int FILE_recv_ascii(FILE *in, FILE *out)
 		foo_hookf(&ftp->ti);
 	ftp->ti.begin = false;
 
-	clearerr(in);
+	sock_clearerr_in(in);
 	clearerr(out);
 
-	while((c = krb_getc(in)) != EOF) {
-		if(ftp_sigints() > 0)
+  bool next = true;
+  int nc = 0;
+	while (true)
+  {
+    int c = 0;
+    if (next)
+    {
+      c = sock_get(in);
+      if (c == EOF)
+        break;
+    }
+    else
+    {
+      c = nc;
+      next = true;
+    }
+
+		if (ftp_sigints() > 0 || wait_for_input() != 0)
 			break;
 
-		if(wait_for_input() != 0)
-			break;
-
-		if(c == '\n')
+		if (c == '\n')
 			ftp->ti.barelfs++;
-		else if(c == '\r') {
-			c = krb_getc(in);
-			if(c == EOF)
+		else if (c == '\r')
+    {
+			nc = sock_get(in);
+			if (nc == EOF)
 				break;
-			if(c != '\n') {
-				ungetc(c, in);
-				c = '\r';
-			}
+			if (nc != '\n')
+        next = false;
+      else
+        c = nc;
 		}
-		if(fputc(c, out) == EOF)
+		if (fputc(c, out) == EOF)
 			break;
 
 		ftp->ti.size++;
@@ -584,15 +598,11 @@ static int FILE_recv_ascii(FILE *in, FILE *out)
 		}
 	}
 
-	free(buf);
-
-	return maybe_abort(in, out);
+	return maybe_abort_in(in, out);
 }
 
-static int FILE_send_ascii(FILE *in, FILE *out)
+static int FILE_send_ascii(FILE* in, Socket* out)
 {
-	char *buf = (char *)xmalloc(FTP_BUFSIZ);
-	int c;
 	time_t then = time(0) - 1;
 	time_t now;
 
@@ -603,8 +613,9 @@ static int FILE_send_ascii(FILE *in, FILE *out)
 	ftp->ti.begin = false;
 
 	clearerr(in);
-	clearerr(out);
+	sock_clearerr_out(out);
 
+  int c;
 	while((c = fgetc(in)) != EOF) {
 		if(ftp_sigints() > 0)
 			break;
@@ -613,11 +624,11 @@ static int FILE_send_ascii(FILE *in, FILE *out)
 			break;
 
 		if(c == '\n') {
-			if(fputc('\r', out) == EOF)
+			if(sock_put(out, '\r') == EOF)
 				break;
 			ftp->ti.size++;
 		}
-		if(fputc(c, out) == EOF)
+		if(sock_put(out, c) == EOF)
 			break;
 		ftp->ti.size++;
 		if(foo_hookf) {
@@ -629,9 +640,7 @@ static int FILE_send_ascii(FILE *in, FILE *out)
 		}
 	}
 
-	free(buf);
-
-	return maybe_abort(in, out);
+	return maybe_abort_out(in, out);
 }
 
 void reset_transfer_info(void)
@@ -654,46 +663,48 @@ void reset_transfer_info(void)
 
 int ftp_list(const char *cmd, const char *param, FILE *fp)
 {
-	if(!cmd || !fp || !ftp_connected())
-		return -1;
+  if (!cmd || !fp || !ftp_connected())
+    return -1;
 
 #ifdef HAVE_LIBSSH
-	if (ftp->session)
-		return ssh_list(cmd, param, fp);
+  if (ftp->session)
+    return ssh_list(cmd, param, fp);
 #endif
 
-	reset_transfer_info();
-	foo_hookf = 0;
+  reset_transfer_info();
+  foo_hookf = NULL;
 
 #if 0 /* don't care about transfer type, binary should work well... */
-	ftp_type(tmAscii);
+  ftp_type(tmAscii);
 #endif
 
-	if(ftp_init_transfer() != 0)
-		return -1;
+  if (ftp_init_transfer() != 0) {
+    ftp_err(_("transfer initialization failed"));
+    return -1;
+  }
 
-	ftp_set_tmp_verbosity(vbNone);
-	if(param)
-		ftp_cmd("%s %s", cmd, param);
-	else
-		ftp_cmd("%s", cmd);
-	if(ftp->code != ctPrelim)
-		return -1;
+  ftp_set_tmp_verbosity(vbNone);
+  if (param)
+    ftp_cmd("%s %s", cmd, param);
+  else
+    ftp_cmd("%s", cmd);
+  if (ftp->code != ctPrelim)
+    return -1;
 
-	if(!sock_accept(ftp->data, "r", ftp_is_passive())) {
-		perror("accept()");
-		return -1;
-	}
+  if (!sock_accept(ftp->data, "r", ftp_is_passive())) {
+    perror("accept()");
+    return -1;
+  }
 
-	if(FILE_recv_ascii(sock_sin(ftp->data), fp) != 0)
-		return -1;
+  if (FILE_recv_ascii(ftp->data, fp) != 0)
+    return -1;
 
-	sock_destroy(ftp->data);
-	ftp->data = NULL;
+  sock_destroy(ftp->data);
+  ftp->data = NULL;
 
-	ftp_read_reply();
+  ftp_read_reply();
 
-	return ftp->code == ctComplete ? 0 : -1;
+  return ftp->code == ctComplete ? 0 : -1;
 }
 
 void transfer_finished(void)
@@ -767,9 +778,9 @@ static int ftp_do_receive(FILE *fp,
 	int r;
 
 	if(mode == tmBinary)
-		r = FILE_recv_binary(sock_sin(ftp->data), fp);
+		r = FILE_recv_binary(ftp->data, fp);
 	else
-		r = FILE_recv_ascii(sock_sin(ftp->data), fp);
+		r = FILE_recv_ascii(ftp->data, fp);
 
 	sock_destroy(ftp->data);
 	ftp->data = 0;
@@ -875,9 +886,9 @@ static int ftp_send(const char *path, FILE *fp, putmode_t how,
 	ftp_cache_flush_mark_for(path);
 
 	if(mode == tmBinary)
-		r = FILE_send_binary(fp, sock_sout(ftp->data));
+		r = FILE_send_binary(fp, ftp->data);
 	else
-		r = FILE_send_ascii(fp, sock_sout(ftp->data));
+		r = FILE_send_ascii(fp, ftp->data);
 	sock_flush(ftp->data);
 	sock_destroy(ftp->data);
 	ftp->data = 0;
@@ -1011,7 +1022,7 @@ int ftp_fxpfile(Ftp *srcftp, const char *srcfile,
 	ftp_cmd("RETR %s", srcfile);
 	if(ftp->code != ctPrelim) {
 		ftp_use(destftp);
-		ftp_abort(0);
+		ftp_abort(NULL);
 		ftp_use(thisftp);
 		return -1;
 	}
